@@ -1,15 +1,24 @@
+#  Licensed under the Apache License, Version 2.0 (the "License"); you may
+#  not use this file except in compliance with the License. You may obtain
+#  a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#  License for the specific language governing permissions and limitations
+#  under the License.
+
 """Application base class.
 """
 
 import itertools
-import logging
-import logging.handlers
 import shlex
+import sys
 
 import cmd2
 
-LOG = logging.getLogger(__name__)
-
 
 class InteractiveApp(cmd2.Cmd):
     """Provides "interactive mode" features.
@@ -34,7 +43,11 @@ class InteractiveApp(cmd2.Cmd):
 
     def __init__(self, parent_app, command_manager, stdin, stdout):
         self.parent_app = parent_app
-        self.prompt = '(%s) ' % parent_app.NAME
+        if not hasattr(sys.stdin, 'isatty') or sys.stdin.isatty():
+            self.prompt = '(%s) ' % parent_app.NAME
+        else:
+            # batch/pipe mode
+            self.prompt = ''
         self.command_manager = command_manager
         cmd2.Cmd.__init__(self, 'tab', stdin=stdin, stdout=stdout)
 
@@ -47,17 +60,33 @@ class InteractiveApp(cmd2.Cmd):
         line_parts = shlex.split(line.parsed.raw)
         self.parent_app.run_subcommand(line_parts)
 
-    def completedefault(self, text, line, begidx, endidx):
-        # Tab-completion for commands known to the command manager.
-        # Does not handle options on the commands.
-        if not text:
-            completions = sorted(n for n, v in self.command_manager)
-        else:
-            completions = sorted(n for n, v in self.command_manager
-                                 if n.startswith(text)
-                                 )
+    def completenames(self, text, line, begidx, endidx):
+        """Tab-completion for command prefix without completer delimiter.
+
+        This method returns cmd style and cliff style commands matching
+        provided command prefix (text).
+        """
+        completions = cmd2.Cmd.completenames(self, text, line, begidx, endidx)
+        completions += self._complete_prefix(text)
         return completions
 
+    def completedefault(self, text, line, begidx, endidx):
+        """Default tab-completion for command prefix with completer delimiter.
+
+        This method filters only cliff style commands matching provided
+        command prefix (line) as cmd2 style commands cannot contain spaces.
+        This method returns text + missing command part of matching commands.
+        This method does not handle options in cmd2/cliff style commands, you
+        must define complete_$method to handle them.
+        """
+        return [x[begidx:] for x in self._complete_prefix(line)]
+
+    def _complete_prefix(self, prefix):
+        """Returns cliff style commands with a specific prefix."""
+        if not prefix:
+            return [n for n, v in self.command_manager]
+        return [n for n, v in self.command_manager if n.startswith(prefix)]
+
     def help_help(self):
         # Use the command manager to get instructions for "help"
         self.default('help help')
@@ -89,6 +118,9 @@ class InteractiveApp(cmd2.Cmd):
             self.print_topics(self.app_cmd_header, cmd_names, 15, 80)
         return
 
+    # Create exit alias to quit the interactive shell.
+    do_exit = cmd2.Cmd.do_quit
+
     def get_names(self):
         # Override the base class version to filter out
         # things that look like they should be hidden
